@@ -4,7 +4,7 @@
 
   Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 2001 - 2003 Pekka Riikonen
+  Copyright (C) 2001 - 2006 Pekka Riikonen
 
   The contents of this file are subject to one of the Licenses specified 
   in the COPYING file;  You may not use this file except in compliance 
@@ -39,6 +39,9 @@
  * SilcHashTableList structure and traverse the hash table inside while()
  * using the list structure. Both are equally fast.
  *
+ * The hash table is not thread safe.  If same hash table context is used in
+ * multi thread environment concurrency control must be employed.
+ *
  ***/
 
 #ifndef SILCHASHTABLE_H
@@ -114,8 +117,8 @@ typedef SilcUInt32 (*SilcHashFunction)(void *key, void *user_context);
  *
  * SYNOPSIS
  *
- *    typedef bool (*SilcHashCompare)(void *key1, void *key2,
- *                                    void *user_context);
+ *    typedef SilcBool (*SilcHashCompare)(void *key1, void *key2,
+ *                                        void *user_context);
  *
  * DESCRIPTION
  *
@@ -126,7 +129,8 @@ typedef SilcUInt32 (*SilcHashFunction)(void *key, void *user_context);
  *    to the callback.
  *
  ***/
-typedef bool (*SilcHashCompare)(void *key1, void *key2, void *user_context);
+typedef SilcBool (*SilcHashCompare)(void *key1, void *key2,
+				    void *user_context);
 
 /****f* silcutil/SilcHashTableAPI/SilcHashDestructor
  *
@@ -175,7 +179,7 @@ typedef void (*SilcHashForeach)(void *key, void *context, void *user_context);
  *                                        void *compare_user_context,
  *                                        SilcHashDestructor destructor,
  *                                        void *destructor_user_context,
- *                                        bool auto_rehash);
+ *                                        SilcBool auto_rehash);
  *
  * DESCRIPTION
  *
@@ -195,7 +199,7 @@ SilcHashTable silc_hash_table_alloc(SilcUInt32 table_size,
 				    void *compare_user_context,
 				    SilcHashDestructor destructor,
 				    void *destructor_user_context,
-				    bool auto_rehash);
+				    SilcBool auto_rehash);
 
 /****f* silcutil/SilcHashTableAPI/silc_hash_table_free
  *
@@ -244,7 +248,7 @@ SilcUInt32 silc_hash_table_count(SilcHashTable ht);
  *
  * SYNOPSIS
  *
- *    void silc_hash_table_add(SilcHashTable ht, void *key, void *context);
+ *    SilcBool silc_hash_table_add(SilcHashTable ht, void *key, void *context);
  *
  * DESCRIPTION
  *
@@ -254,13 +258,14 @@ SilcUInt32 silc_hash_table_count(SilcHashTable ht);
  *    to the hash table reliably (it is collision resistant).
  *
  ***/
-void silc_hash_table_add(SilcHashTable ht, void *key, void *context);
+SilcBool silc_hash_table_add(SilcHashTable ht, void *key, void *context);
 
 /****f* silcutil/SilcHashTableAPI/silc_hash_table_replace
  *
  * SYNOPSIS
  *
- *    void silc_hash_table_replace(SilcHashTable ht, void *key, void *context);
+ *    SilcBool silc_hash_table_replace(SilcHashTable ht, void *key,
+ *                                     void *context);
  *
  * DESCRIPTION
  *
@@ -270,13 +275,13 @@ void silc_hash_table_add(SilcHashTable ht, void *key, void *context);
  *    replaced key and context.
  *
  ***/
-void silc_hash_table_replace(SilcHashTable ht, void *key, void *context);
+SilcBool silc_hash_table_replace(SilcHashTable ht, void *key, void *context);
 
 /****f* silcutil/SilcHashTableAPI/silc_hash_table_del
  *
  * SYNOPSIS
  *
- *    bool silc_hash_table_del(SilcHashTable ht, void *key);
+ *    SilcBool silc_hash_table_del(SilcHashTable ht, void *key);
  *
  * DESCRIPTION
  *
@@ -285,14 +290,14 @@ void silc_hash_table_replace(SilcHashTable ht, void *key, void *context);
  *    entry was removed successfully and FALSE otherwise.
  *
  ***/
-bool silc_hash_table_del(SilcHashTable ht, void *key);
+SilcBool silc_hash_table_del(SilcHashTable ht, void *key);
 
 /****f* silcutil/SilcHashTableAPI/silc_hash_table_del_by_context
  *
  * SYNOPSIS
  *
- *    bool silc_hash_table_del_by_context(SilcHashTable ht, void *key,
- *                                        void *context);
+ *    SilcBool silc_hash_table_del_by_context(SilcHashTable ht, void *key,
+ *                                            void *context);
  *
  * DESCRIPTION
  *
@@ -302,15 +307,15 @@ bool silc_hash_table_del(SilcHashTable ht, void *key);
  *    be used to check whether the correct entry is being deleted.
  *
  ***/
-bool silc_hash_table_del_by_context(SilcHashTable ht, void *key,
-				    void *context);
+SilcBool silc_hash_table_del_by_context(SilcHashTable ht, void *key,
+					void *context);
 
 /****f* silcutil/SilcHashTableAPI/silc_hash_table_find
  *
  * SYNOPSIS
  *
- *    bool silc_hash_table_find(SilcHashTable ht, void *key,
- *                              void **ret_key, void **ret_context);
+ *    SilcBool silc_hash_table_find(SilcHashTable ht, void *key,
+ *                                  void **ret_key, void **ret_context);
  *
  * DESCRIPTION
  *
@@ -321,15 +326,15 @@ bool silc_hash_table_del_by_context(SilcHashTable ht, void *key,
  *    maybe used only to check whether given key exists in the table.
  *
  ***/
-bool silc_hash_table_find(SilcHashTable ht, void *key,
-			  void **ret_key, void **ret_context);
+SilcBool silc_hash_table_find(SilcHashTable ht, void *key,
+			      void **ret_key, void **ret_context);
 
 /****f* silcutil/SilcHashTableAPI/silc_hash_table_find_by_context
  *
  * SYNOPSIS
  *
- *    bool silc_hash_table_find_by_context(SilcHashTable ht, void *key,
- *                                         void *context, void **ret_key);
+ *    SilcBool silc_hash_table_find_by_context(SilcHashTable ht, void *key,
+ *                                             void *context, void **ret_key);
  *
  * DESCRIPTION
  *
@@ -342,8 +347,8 @@ bool silc_hash_table_find(SilcHashTable ht, void *key,
  *    the caller already knows the context.
  *
  ***/
-bool silc_hash_table_find_by_context(SilcHashTable ht, void *key,
-				     void *context, void **ret_key);
+SilcBool silc_hash_table_find_by_context(SilcHashTable ht, void *key,
+					 void *context, void **ret_key);
 
 /****f* silcutil/SilcHashTableAPI/silc_hash_table_find_foreach
  *
@@ -450,17 +455,26 @@ void silc_hash_table_list_reset(SilcHashTableList *htl);
  *
  * SYNOPSIS
  *
- *    bool silc_hash_table_get(SilcHashTableList *htl, void **key,
- *                             void **context);
+ *    SilcBool silc_hash_table_get(SilcHashTableList *htl, void **key,
+ *                                 void **context);
  *
  * DESCRIPTION
  *
  *    Returns always the next entry in the hash table into the `key' and
- *    `context' and TRUE.  If this returns FALSE then there are no anymore
- *    any entrys.
+ *    `context' and TRUE.  If this returns FALSE then there are no more
+ *    entries.
+ *
+ * EXAMPLE
+ *
+ *    SilcHashTableList htl;
+ *    silc_hash_table_list(hash_table, &htl);
+ *    while (silc_hash_table_get(&htl, (void *)&key, (void *)&context))
+ *      ...
+ *    silc_hash_table_list_reset(&htl);
  *
  ***/
-bool silc_hash_table_get(SilcHashTableList *htl, void **key, void **context);
+SilcBool silc_hash_table_get(SilcHashTableList *htl,
+			     void **key, void **context);
 
 
 /* Extended hash table interface (same as above but with specific
@@ -470,9 +484,10 @@ bool silc_hash_table_get(SilcHashTableList *htl, void **key, void **context);
  *
  * SYNOPSIS
  *
- *    void silc_hash_table_add_ext(SilcHashTable ht, void *key, void *context,
- *                                 SilcHashFunction hash,
- *                                 void *hash_user_context);
+ *    SilcBool silc_hash_table_add_ext(SilcHashTable ht, void *key,
+ *                                     void *context,
+ *                                     SilcHashFunction hash,
+ *                                     void *hash_user_context);
  *
  * DESCRIPTION
  *
@@ -485,17 +500,19 @@ bool silc_hash_table_get(SilcHashTableList *htl, void **key, void **context);
  *    function. If not provided the hash table's default is used.
  *
  ***/
-void silc_hash_table_add_ext(SilcHashTable ht, void *key, void *context,
-			     SilcHashFunction hash, void *hash_user_context);
+SilcBool silc_hash_table_add_ext(SilcHashTable ht,
+				 void *key, void *context,
+				 SilcHashFunction hash,
+				 void *hash_user_context);
 
 /****f* silcutil/SilcHashTableAPI/silc_hash_table_replace_ext
  *
  * SYNOPSIS
  *
- *    void silc_hash_table_replace_ext(SilcHashTable ht, void *key,
- *                                     void *context,
- *                                     SilcHashFunction hash,
- *                                     void *hash_user_context);
+ *    SilcBool silc_hash_table_replace_ext(SilcHashTable ht, void *key,
+ *                                         void *context,
+ *                                         SilcHashFunction hash,
+ *                                         void *hash_user_context);
  *
  * DESCRIPTION
  *
@@ -508,21 +525,22 @@ void silc_hash_table_add_ext(SilcHashTable ht, void *key, void *context,
  *    function. If not provided the hash table's default is used.
  *
  ***/
-void silc_hash_table_replace_ext(SilcHashTable ht, void *key, void *context,
-				 SilcHashFunction hash,
-				 void *hash_user_context);
+SilcBool silc_hash_table_replace_ext(SilcHashTable ht,
+				     void *key, void *context,
+				     SilcHashFunction hash,
+				     void *hash_user_context);
 
 /****f* silcutil/SilcHashTableAPI/silc_hash_table_del_ext
  *
  * SYNOPSIS
  *
- *    bool silc_hash_table_del_ext(SilcHashTable ht, void *key,
- *                                 SilcHashFunction hash,
- *                                 void *hash_user_context,
- *                                 SilcHashCompare compare,
- *                                 void *compare_user_context,
- *                                 SilcHashDestructor destructor,
- *                                 void *destructor_user_context);
+ *    SilcBool silc_hash_table_del_ext(SilcHashTable ht, void *key,
+ *                                     SilcHashFunction hash,
+ *                                     void *hash_user_context,
+ *                                     SilcHashCompare compare,
+ *                                     void *compare_user_context,
+ *                                     SilcHashDestructor destructor,
+ *                                     void *destructor_user_context);
  *
  * DESCRIPTION
  *
@@ -538,26 +556,27 @@ void silc_hash_table_replace_ext(SilcHashTable ht, void *key, void *context,
  *    specific destructor function.
  *
  ***/
-bool silc_hash_table_del_ext(SilcHashTable ht, void *key,
-			     SilcHashFunction hash,
-			     void *hash_user_context,
-			     SilcHashCompare compare,
-			     void *compare_user_context,
-			     SilcHashDestructor destructor,
-			     void *destructor_user_context);
+SilcBool silc_hash_table_del_ext(SilcHashTable ht, void *key,
+				 SilcHashFunction hash,
+				 void *hash_user_context,
+				 SilcHashCompare compare,
+				 void *compare_user_context,
+				 SilcHashDestructor destructor,
+				 void *destructor_user_context);
 
 /****f* silcutil/SilcHashTableAPI/silc_hash_table_del_by_context_ext
  *
  * SYNOPSIS
  *
- *    bool silc_hash_table_del_by_context_ext(SilcHashTable ht, void *key,
- *                                            void *context,
- *                                            SilcHashFunction hash,
- *                                            void *hash_user_context,
- *                                            SilcHashCompare compare,
- *                                            void *compare_user_context,
- *                                            SilcHashDestructor destructor,
- *                                            void *destructor_user_context);
+ *    SilcBool
+ *    silc_hash_table_del_by_context_ext(SilcHashTable ht, void *key,
+ *                                       void *context,
+ *                                       SilcHashFunction hash,
+ *                                       void *hash_user_context,
+ *                                       SilcHashCompare compare,
+ *                                       void *compare_user_context,
+ *                                       SilcHashDestructor destructor,
+ *                                       void *destructor_user_context);
  *
  * DESCRIPTION
  *
@@ -574,25 +593,25 @@ bool silc_hash_table_del_ext(SilcHashTable ht, void *key,
  *    specific destructor function.
  *
  ***/
-bool silc_hash_table_del_by_context_ext(SilcHashTable ht, void *key,
-					void *context,
-					SilcHashFunction hash,
-					void *hash_user_context,
-					SilcHashCompare compare,
-					void *compare_user_context,
-					SilcHashDestructor destructor,
-					void *destructor_user_context);
+SilcBool silc_hash_table_del_by_context_ext(SilcHashTable ht, void *key,
+					    void *context,
+					    SilcHashFunction hash,
+					    void *hash_user_context,
+					    SilcHashCompare compare,
+					    void *compare_user_context,
+					    SilcHashDestructor destructor,
+					    void *destructor_user_context);
 
 /****f* silcutil/SilcHashTableAPI/silc_hash_table_find_ext
  *
  * SYNOPSIS
  *
- *    bool silc_hash_table_find_ext(SilcHashTable ht, void *key,
- *                                  void **ret_key, void **ret_context,
- *                                  SilcHashFunction hash,
- *                                  void *hash_user_context,
- *                                  SilcHashCompare compare,
- *                                  void *compare_user_context);
+ *    SilcBool silc_hash_table_find_ext(SilcHashTable ht, void *key,
+ *                                      void **ret_key, void **ret_context,
+ *                                      SilcHashFunction hash,
+ *                                      void *hash_user_context,
+ *                                      SilcHashCompare compare,
+ *                                      void *compare_user_context);
  *
  * DESCRIPTION
  *
@@ -608,23 +627,24 @@ bool silc_hash_table_del_by_context_ext(SilcHashTable ht, void *key,
  *    comparing function. If not provided the hash table's default is used.
  *
  ***/
-bool silc_hash_table_find_ext(SilcHashTable ht, void *key,
-			      void **ret_key, void **ret_context,
-			      SilcHashFunction hash,
-			      void *hash_user_context,
-			      SilcHashCompare compare,
-			      void *compare_user_context);
+SilcBool silc_hash_table_find_ext(SilcHashTable ht, void *key,
+				  void **ret_key, void **ret_context,
+				  SilcHashFunction hash,
+				  void *hash_user_context,
+				  SilcHashCompare compare,
+				  void *compare_user_context);
 
 /****f* silcutil/SilcHashTableAPI/silc_hash_table_find_by_context_ext
  *
  * SYNOPSIS
  *
- *    bool silc_hash_table_find_by_context_ext(SilcHashTable ht, void *key,
- *                                             void *context, void **ret_key,
- *                                             SilcHashFunction hash,
- *                                             void *hash_user_context,
- *                                             SilcHashCompare compare,
- *                                             void *compare_user_context);
+ *    SilcBool
+ *    silc_hash_table_find_by_context_ext(SilcHashTable ht, void *key,
+ *                                        void *context, void **ret_key,
+ *                                        SilcHashFunction hash,
+ *                                        void *hash_user_context,
+ *                                        SilcHashCompare compare,
+ *                                        void *compare_user_context);
  *
  * DESCRIPTION
  *
@@ -642,12 +662,12 @@ bool silc_hash_table_find_ext(SilcHashTable ht, void *key,
  *    comparing function. If not provided the hash table's default is used.
  *
  ***/
-bool silc_hash_table_find_by_context_ext(SilcHashTable ht, void *key,
-					 void *context, void **ret_key,
-					 SilcHashFunction hash,
-					 void *hash_user_context,
-					 SilcHashCompare compare,
-					 void *compare_user_context);
+SilcBool silc_hash_table_find_by_context_ext(SilcHashTable ht, void *key,
+					     void *context, void **ret_key,
+					     SilcHashFunction hash,
+					     void *hash_user_context,
+					     SilcHashCompare compare,
+					     void *compare_user_context);
 
 /****f* silcutil/SilcHashTableAPI/silc_hash_table_find_foreach_ext
  *
