@@ -24,6 +24,12 @@
 #include <windows.h>
 #include <io.h>
 #include <process.h>
+#include <fcntl.h>
+#include <assert.h>
+
+#if defined(HAVE_SILCDEFS_H)
+#include "regexpr.h"
+#endif /* HAVE_SILCDEFS_H */
 
 #define snprintf _snprintf
 #define vsnprintf _vsnprintf
@@ -65,4 +71,9 @@
 #undef sleep
 #define sleep(x) Sleep((x) * 1000)
 
+#if _MSC_VER < 1300
+#define SetWindowLongPtr SetWindowLong
+#define GetWindowLongPtr GetWindowLong
+#endif /* _MSC_VER < 1300 */
+
 #endif
