@@ -1,6 +1,6 @@
 /*
 
-  stacktrace.h 
+  stacktrace.h
 
   Author: Pekka Riikonen <priikone@silcnet.org>
 
@@ -17,14 +17,14 @@
 
 */
 
-#ifndef STACKTRACE_H
-#define STACKTRACE_H
+#ifndef MEMTRACE_H
+#define MEMTRACE_H
 
 #ifndef SILCMEMORY_H
 #error "Do not include internal header file directly"
 #endif
 
-#if defined(__GNUC__) && defined(__i386__)
+#if defined(__GNUC__)
 
 #undef strdup
 #define silc_malloc(s)      silc_st_malloc((s), __FILE__, __LINE__)
@@ -32,6 +32,7 @@
 #define silc_realloc(p, s)  silc_st_realloc((p), (s), __FILE__, __LINE__)
 #define silc_free(p)        silc_st_free((p), __FILE__, __LINE__)
 #define silc_memdup(p, s)   silc_st_memdup((p), (s), __FILE__, __LINE__)
+#define silc_strdup(s)      silc_st_strdup((s), __FILE__, __LINE__)
 #define strdup(s)           silc_st_strdup((s), __FILE__, __LINE__)
 
 void *silc_st_malloc(size_t size, const char *file, int line);
@@ -44,6 +45,6 @@ void silc_st_dump(void);
 
 #else
 #error "memory allocation stack trace not supported on this platform"
-#endif /* __GNUC__ && __i386__ */
+#endif /* __GNUC__ */
 
-#endif /* STACKTRACE_H */
+#endif /* MEMTRACE_H */
